@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,6 +17,7 @@
  *
  */
 
+#include <assert.h>
 #include <errno.h>
 #include <math.h>
 #include <pthread.h>
@@ -100,8 +101,7 @@
 
 #define FUNC_LOAD 0
 #define FUNC_RECUE 1
-#define FUNC_DISCONNECT 2
-#define FUNC_RECONNECT 3
+#define FUNC_TIMECODE 2
 
 
 /* State variables used to trigger certain actions */
@@ -110,6 +110,12 @@
 #define UPDATE_REDRAW 1
 
 
+/* Types of SDL_USEREVENT */
+
+#define EVENT_TICKER 0
+#define EVENT_QUIT 1
+
+
 /* Macro functions */
 
 #define MIN(x,y) ((x)<(y)?(x):(y))
@@ -374,9 +380,10 @@ static int load_fonts(void)
 
 static void clear_fonts(void)
 {
+    TTF_CloseFont(clock_font);
+    TTF_CloseFont(deci_font);
     TTF_CloseFont(font);
     TTF_CloseFont(em_font);
-    TTF_CloseFont(clock_font);
     TTF_CloseFont(detail_font);
 }
 
@@ -796,7 +803,7 @@ static void draw_deck_top(SDL_Surface *surface, const struct rect_t *rect,
     /* If there is no timecoder to display information on, or not enough 
      * available space, just draw clocks which span the overall space */
 
-    if (!pl->timecoder || right.w < 0) {
+    if (!pl->timecode_control || right.w < 0) {
         draw_deck_clocks(surface, rect, pl);
         return;
     }
@@ -826,8 +833,9 @@ static void draw_deck_status(SDL_Surface *surface,
 {
     char buf[128];
     int tc;
-    
-    if (pl->timecoder && (tc=timecoder_get_position(pl->timecoder, NULL)) != -1)
+
+    tc = timecoder_get_position(pl->timecoder, NULL);
+    if (pl->timecode_control && tc != -1)
         sprintf(buf, "timecode:%d      ", tc);
     else
         sprintf(buf, "timecode:        ");
@@ -837,7 +845,7 @@ static void draw_deck_status(SDL_Surface *surface,
             pl->sync_pitch,
             pl->last_difference,
             pl->pitch * pl->sync_pitch,
-            pl->reconnect ? "RECN  " : "");
+            pl->recalibrate ? "RCAL  " : "");
     
     draw_font_rect(surface, rect, buf, detail_font,
                    detail_col, background_col);
@@ -934,7 +942,7 @@ static void draw_search(SDL_Surface *surface, const struct rect_t *rect,
     SDL_FillRect(surface, &cursor, palette(surface, &cursor_col));
 
     if (sel->view_listing->entries > 1)
-        sprintf(cm, "%d matches", sel->view_listing->entries);
+        sprintf(cm, "%zd matches", sel->view_listing->entries);
     else if (sel->view_listing->entries > 0)
         sprintf(cm, "1 match");
     else
@@ -1131,14 +1139,18 @@ static void draw_library(SDL_Surface *surface, const struct rect_t *rect,
 /* Initiate the process of loading a record from the library into a
  * track in memory */
 
-static void do_loading(struct track_t *track, struct record_t *record)
+static void do_loading(struct interface_t *interface,
+                       struct track_t *track, struct record_t *record)
 {
     fprintf(stderr, "Loading '%s'.\n", record->pathname);
 
-    track_import(track, record->pathname);
+    if (track_import(track, record->pathname) == -1)
+        return;
 
     track->artist = record->artist;
     track->title = record->title;
+
+    (void)rig_awaken(interface->rig);
 }
 
 
@@ -1244,19 +1256,15 @@ static bool handle_key(struct interface_t *in, struct selector_t *sel,
             case FUNC_LOAD:
                 re = selector_current(sel);
                 if (re != NULL)
-                    do_loading(pl->track, re);
+                    do_loading(in, pl->track, re);
                 break;
 
             case FUNC_RECUE:
                 player_recue(pl);
                 break;
 
-            case FUNC_DISCONNECT:
-                player_disconnect_timecoder(pl);
-                break;
-
-            case FUNC_RECONNECT:
-                player_connect_timecoder(pl, in->timecoder[deck]);
+            case FUNC_TIMECODE:
+                (void)player_toggle_timecode_control(pl);
                 break;
             }
         }
@@ -1294,6 +1302,7 @@ static Uint32 ticker(Uint32 interval, void *p)
     if (!SDL_PeepEvents(&event, 1, SDL_PEEKEVENT, SDL_EVENTMASK(SDL_USEREVENT)))
     {
         event.type = SDL_USEREVENT;
+        event.user.code = EVENT_TICKER;
         SDL_PushEvent(&event);
     }        
 
@@ -1301,64 +1310,23 @@ static Uint32 ticker(Uint32 interval, void *p)
 }
 
 
-void interface_init(struct interface_t *in)
+/*
+ * The SDL interface thread
+ */
+
+static int interface_main(struct interface_t *in)
 {
-    int n;
-
-    for (n = 0; n < MAX_PLAYERS; n++)
-        in->player[n] = NULL;
-
-    in->library = NULL;
-}
-
-
-int interface_run(struct interface_t *in)
-{
-    int meter_scale, p, finished,
-        library_update, decks_update, status_update;
-    const char *status = BANNER;
+    int meter_scale, library_update, decks_update, status_update;
+    const char *status = banner;
 
     SDL_Event event;
     SDL_TimerID timer;
     SDL_Surface *surface;
 
     struct rect_t rworkspace, rplayers, rlibrary, rstatus, rtmp;
-    struct selector_t selector;
 
-    /* Initialise our own data structures */
-
-    finished = 0;
     meter_scale = DEFAULT_METER_SCALE;
 
-    for (p = 0; p < in->timecoders; p++) {
-        if (timecoder_monitor_init(in->timecoder[p], SCOPE_SIZE) == -1)
-            return -1;
-    }
-
-    selector_init(&selector, in->library);
-    calculate_spinner_lookup(spinner_angle, NULL, SPINNER_SIZE);
-
-    fprintf(stderr, "Initialising SDL...\n");
-    
-    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) == -1) {
-        fprintf(stderr, "%s\n", SDL_GetError());
-        return -1;
-    }
-    SDL_WM_SetCaption(BANNER, NULL);
-    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
-
-    /* Initialise the fonts */
-    
-    if (TTF_Init() == -1) {
-        fprintf(stderr, "%s\n", TTF_GetError());
-        return -1;
-    }
-
-    if (load_fonts() == -1)
-        return -1;
-
-    /* Initialise the screen surface */
-
     surface = set_size(DEFAULT_WIDTH, DEFAULT_HEIGHT, &rworkspace);
     if (!surface)
         return -1;
@@ -1371,12 +1339,12 @@ int interface_run(struct interface_t *in)
 
     timer = SDL_AddTimer(REFRESH, ticker, (void*)in);
 
-    while (!finished && SDL_WaitEvent(&event) >= 0) {
+    while (SDL_WaitEvent(&event) >= 0) {
 
         switch(event.type) {
-            
-        case SDL_QUIT:
-            finished = 1;
+        case SDL_QUIT: /* user request to quit application; eg. window close */
+            if (rig_quit(in->rig) == -1)
+                return -1;
             break;
 
         case SDL_VIDEORESIZE:
@@ -1395,15 +1363,23 @@ int interface_run(struct interface_t *in)
 
             break;
             
-        case SDL_USEREVENT: /* request to update the clocks */
+        case SDL_USEREVENT:
+            switch (event.user.code) {
+            case EVENT_TICKER: /* request to poll the clocks */
+                if (decks_update < UPDATE_REDRAW)
+                    decks_update = UPDATE_REDRAW;
+                break;
 
-            if (decks_update < UPDATE_REDRAW)
-                decks_update = UPDATE_REDRAW;
-            
+            case EVENT_QUIT: /* internal request to finish this thread */
+                goto finish;
+
+            default:
+                abort();
+            }
             break;
 
         case SDL_KEYDOWN:
-            if (handle_key(in, &selector, &meter_scale,
+            if (handle_key(in, &in->selector, &meter_scale,
                           event.key.keysym.sym, event.key.keysym.mod))
             {
                 library_update = UPDATE_REDRAW;
@@ -1434,14 +1410,14 @@ int interface_run(struct interface_t *in)
 
         if (library_update >= UPDATE_REDRAW) {
 
-            if (selector_current(&selector) != NULL)
-                status = selector_current(&selector)->pathname;
+            if (selector_current(&in->selector) != NULL)
+                status = selector_current(&in->selector)->pathname;
             else
                 status = "No search results found";
             status_update = UPDATE_REDRAW;
           
             LOCK(surface);
-            draw_library(surface, &rlibrary, &selector);
+            draw_library(surface, &rlibrary, &in->selector);
             UNLOCK(surface);
             UPDATE(surface, &rlibrary);
             library_update = UPDATE_NONE;
@@ -1471,17 +1447,100 @@ int interface_run(struct interface_t *in)
 
     } /* main loop */
 
+ finish:
     SDL_RemoveTimer(timer);
 
-    for (p = 0; p < in->timecoders; p++)
-        timecoder_monitor_clear(in->timecoder[p]);
+    return 0;
+}
 
-    selector_clear(&selector);
+static void* launch(void *p)
+{
+    interface_main(p);
+    return NULL;
+}
+
+/*
+ * Start the SDL interface
+ *
+ * FIXME: There are multiple points where resources are leaked on
+ * error
+ */
+
+int interface_start(struct interface_t *in, size_t ndeck,
+                    struct player_t *pl, struct timecoder_t *tc,
+                    struct library_t *lib, struct rig_t *rig)
+{
+    size_t n;
+
+    assert(ndeck <= MAX_DECKS);
+
+    for (n = 0; n < ndeck; n++) {
+        in->player[n] = &pl[n];
+        in->timecoder[n] = &tc[n];
+        if (timecoder_monitor_init(in->timecoder[n], SCOPE_SIZE) == -1)
+            return -1;
+    }
+
+    in->players = ndeck;
+    in->timecoders = ndeck;
+
+    in->rig = rig;
+    selector_init(&in->selector, lib);
+    calculate_spinner_lookup(spinner_angle, NULL, SPINNER_SIZE);
+
+    fprintf(stderr, "Initialising SDL...\n");
+
+    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) == -1) {
+        fprintf(stderr, "%s\n", SDL_GetError());
+        return -1;
+    }
+    SDL_WM_SetCaption(banner, NULL);
+    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
+
+    /* Initialise the fonts */
+
+    if (TTF_Init() == -1) {
+        fprintf(stderr, "%s\n", TTF_GetError());
+        return -1;
+    }
+
+    if (load_fonts() == -1)
+        return -1;
+
+    fprintf(stderr, "Launching interface thread...\n");
+
+    if (pthread_create(&in->ph, NULL, launch, (void*)in)) {
+        perror("pthread_create");
+        return -1;
+    }
+
+    return 0;
+}
+
+/*
+ * Synchronise with the SDL interface and exit
+ */
+
+void interface_stop(struct interface_t *in)
+{
+    size_t n;
+    SDL_Event quit;
+
+    quit.type = SDL_USEREVENT;
+    quit.user.code = EVENT_QUIT;
+    if (SDL_PushEvent(&quit) == -1)
+        abort();
+
+    if (pthread_join(in->ph, NULL) != 0)
+        abort();
+
+    for (n = 0; n < in->timecoders; n++)
+        timecoder_monitor_clear(in->timecoder[n]);
+
+    selector_clear(&in->selector);
 
     clear_fonts();
 
     TTF_Quit();
     SDL_Quit();
-    
-    return 0;
 }
