@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,135 +31,109 @@
 
 #include "library.h"
 
+#define CRATE_ALL "All records"
 
-int library_init(struct library_t *li)
+/*
+ * Initialise a crate
+ *
+ * Note the deep copy of the crate name
+ *
+ * Return: 0 on success or -1 on memory allocation failure
+ */
+
+static int crate_init(struct crate_t *c, const char *name, bool is_fixed)
 {
-    listing_init(&li->storage);
-
-    li->crate = malloc(sizeof(struct crate_t*));
-    if (li->crate == NULL) {
-        perror("malloc");
+    c->name = strdup(name);
+    if (c->name == NULL) {
+        perror("strdup");
         return -1;
     }
 
-    li->crates = 0;
-
-    library_new_crate(li, CRATE_ALL, true);
+    c->is_fixed = is_fixed;
+    listing_init(&c->listing);
 
     return 0;
 }
 
+/*
+ * Deallocate resources associated with this crate
+ *
+ * The crate does not 'own' the memory allocated by the records
+ * in it, so we don't free them here.
+ */
 
-static void record_clear(struct record_t *re)
+static void crate_clear(struct crate_t *c)
 {
-    free(re->pathname);
-    free(re->artist);
-    free(re->title);
+    listing_clear(&c->listing);
+    free(c->name);
 }
 
+/*
+ * Comparison function for two crates
+ */
 
-void library_clear(struct library_t *li)
+static int crate_cmp(const struct crate_t *a, const struct crate_t *b)
 {
-    int n;
+    if (a->is_fixed && !b->is_fixed)
+        return -1;
+    if (!a->is_fixed && b->is_fixed)
+        return 1;
 
-    for (n = 0; n < li->crates; n++) {
-        struct crate_t *crate;
-
-        crate = li->crate[n];
-        listing_clear(&crate->listing);
-        free(crate->name);
-        free(crate);
-    }
-    free(li->crate);
-
-    /* This object owns all the record pointers */
-
-    for (n = 0; n < li->storage.entries; n++) {
-        struct record_t *re;
-
-        re = li->storage.record[n];
-        record_clear(re);
-        free(re);
-    }
-
-    listing_clear(&li->storage);
+    return strcmp(a->name, b->name);
 }
 
+/*
+ * Comparison function, see qsort(3)
+ */
 
-static void library_swap_crates(struct library_t *lib, int i, int j)
+static int qcompar(const void *a, const void *b)
 {
-    struct crate_t *tmp;
-    tmp = lib->crate[i];
-    lib->crate[i] = lib->crate[j];
-    lib->crate[j] = tmp;
+    return crate_cmp(*(struct crate_t**)a, *(struct crate_t**)b);
 }
 
+/*
+ * Sort all crates into a defined order
+ */
 
-static void library_sort_crates(struct library_t *lib)
+static void sort_crates(struct library_t *lib)
 {
-    int i, changed;
-
-    do {
-        changed = 0;
-
-        for (i = 0; i < lib->crates - 1; i++) {
-
-            if (lib->crate[i]->is_fixed)
-                continue;
-
-            if (lib->crate[i+1]->is_fixed) {
-                library_swap_crates(lib, i, i+1);
-                changed++;
-                continue;
-            }
-
-            if (strcmp(lib->crate[i]->name, lib->crate[i + 1]->name) > 0) {
-                library_swap_crates(lib, i, i+1);
-                changed++;
-            }
-
-        }
-    } while (changed);
+    qsort(lib->crate, lib->crates, sizeof(struct crate_t*), qcompar);
 }
 
+/*
+ * Add a crate to the list of all crates
+ *
+ * Return: 0 on success or -1 on memory allocation failure
+ */
 
-struct crate_t* library_new_crate(struct library_t *lib, char *name,
-                                  bool is_fixed)
+static int add_crate(struct library_t *lib, struct crate_t *c)
 {
-    struct crate_t *new_crate;
-
-    /* does this crate already exist? then return existing crate */
-    new_crate = library_get_crate(lib, name);
-    if (new_crate != NULL) {
-        fprintf(stderr, "Crate '%s' already exists...\n", name);
-        return new_crate;
-    }
-
-    /* allocate and fill space for new crate */
-    new_crate = malloc(sizeof(struct crate_t));
-    new_crate->name = strdup(name);
-    new_crate->is_fixed = is_fixed;
-
-    listing_init(&new_crate->listing);
-
-    /* add this new crate to the library */
     struct crate_t **cn;
+
     cn = realloc(lib->crate, sizeof(struct crate_t*) * (lib->crates + 1));
-    if (!cn) {
+    if (cn == NULL) {
         perror("realloc");
-        return NULL;
+        return -1;
     }
 
     lib->crate = cn;
-    lib->crate[lib->crates++] = new_crate;
+    lib->crate[lib->crates++] = c;
 
-    library_sort_crates(lib);
+    sort_crates(lib);
 
-    return new_crate;
+    return 0;
 }
 
+/*
+ * Get a crate by the given name
+ *
+ * Beware: The match could match the fixed crates if the name is the
+ * same.
+ *
+ * Return: pointer to crate, or NULL if no crate has the given name
+ */
 
-struct crate_t* library_get_crate(struct library_t *lib, char *name)
+struct crate_t* get_crate(struct library_t *lib, const char *name)
 {
     int n;
 
@@ -171,16 +145,118 @@ struct crate_t* library_get_crate(struct library_t *lib, char *name)
     return NULL;
 }
 
+/*
+ * Get an existing crate, or create a new one if necessary
+ *
+ * Return: pointer to crate, or NULL on memory allocation failure
+ */
 
-static int crate_add(struct crate_t *crate, struct record_t *lr)
+struct crate_t* use_crate(struct library_t *lib, char *name, bool is_fixed)
 {
-    return listing_add(&crate->listing, lr);
+    struct crate_t *new_crate;
+
+    /* does this crate already exist? then return existing crate */
+    new_crate = get_crate(lib, name);
+    if (new_crate != NULL) {
+        fprintf(stderr, "Crate '%s' already exists...\n", name);
+        return new_crate;
+    }
+
+    /* allocate and fill space for new crate */
+    new_crate = malloc(sizeof(struct crate_t));
+    if (new_crate == NULL) {
+        perror("malloc");
+        return NULL;
+    }
+
+    if (crate_init(new_crate, name, is_fixed) == -1)
+        goto fail;
+
+    if (add_crate(lib, new_crate) == -1)
+        goto fail;
+
+    return new_crate;
+
+ fail:
+    free(new_crate);
+    return NULL;
 }
 
+/*
+ * Initialise the record library
+ *
+ * Return: 0 on success or -1 on memory allocation failure
+ */
 
-/* Read and allocate a null-terminated field from the given file handle.
- * If the empty string is read, *s is set to NULL. Return 0 on success,
- * -1 on error or EOF */
+int library_init(struct library_t *li)
+{
+    li->crate = NULL;
+    li->crates = 0;
+
+    if (crate_init(&li->all, CRATE_ALL, true) == -1)
+        return -1;
+
+    if (add_crate(li, &li->all) == -1) {
+        crate_clear(&li->all);
+        return -1;
+    }
+
+    return 0;
+}
+
+/*
+ * Free resources associated with a record
+ */
+
+static void record_clear(struct record_t *re)
+{
+    free(re->pathname);
+    free(re->artist);
+    free(re->title);
+}
+
+/*
+ * Free resources associated with the music library
+ */
+
+void library_clear(struct library_t *li)
+{
+    int n;
+
+    /* This object is responsible for all the record pointers */
+
+    for (n = 0; n < li->all.listing.entries; n++) {
+        struct record_t *re;
+
+        re = li->all.listing.record[n];
+        record_clear(re);
+        free(re);
+    }
+
+    /* Clear crates */
+
+    for (n = 1; n < li->crates; n++) { /* skip the 'all' crate */
+        struct crate_t *crate;
+
+        crate = li->crate[n];
+        crate_clear(crate);
+        free(crate);
+    }
+    free(li->crate);
+
+    crate_clear(&li->all);
+}
+
+/*
+ * Read a string from a file.
+ *
+ * A null-terminated string is read from the given file handle
+ * by reading until the delimiter.
+ *
+ * Return: 0 on success, -1 on error or EOF
+ * Post: On success, *s is an alloc'd string returned to the caller;
+ *     if an empty string is read, *s is set to NULL
+ */
 
 static int get_field(FILE *fp, char delim, char **f)
 {
@@ -198,11 +274,17 @@ static int get_field(FILE *fp, char delim, char **f)
     return 0;
 }
 
+/*
+ * Scan a record library
+ *
+ * Launch the given scan script and pass it the path argument.
+ * Parse the results into the crates.
+ *
+ * Return: 0 on success, -1 on fatal error (may leak)
+ */
 
-/* Scan a record library at the given path, using the given scan
- * script. Returns -1 on fatal error which may leak resources */
-
-int library_import(struct library_t *li, const char *scan, const char *path)
+int library_import(struct library_t *li, bool sort,
+                   const char *scan, const char *path)
 {
     int pstdout[2], status;
     char *cratename, *pathname;
@@ -212,16 +294,13 @@ int library_import(struct library_t *li, const char *scan, const char *path)
 
     fprintf(stderr, "Scanning '%s'...\n", path);
 
-    all_crate = library_get_crate(li, CRATE_ALL);
-    if (all_crate == NULL) {
-        fprintf(stderr, "Could not get ALL_CRATE..");
-        return -1;
-    }
+    all_crate = get_crate(li, CRATE_ALL);
+    assert(all_crate != NULL);
 
     pathname = strdupa(path);
     cratename = basename(pathname); /* POSIX version, see basename(3) */
     assert(cratename != NULL);
-    crate = library_new_crate(li, cratename, false);
+    crate = use_crate(li, cratename, false);
     if (crate == NULL)
         return -1;
 
@@ -274,7 +353,7 @@ int library_import(struct library_t *li, const char *scan, const char *path)
     }
 
     for (;;) {
-        struct record_t *d;
+        struct record_t *d, *x;
         char *pathname;
 
         if (get_field(fp, '\t', &pathname) != 0)
@@ -298,15 +377,29 @@ int library_import(struct library_t *li, const char *scan, const char *path)
             return -1;
         }
 
-        if (listing_add(&li->storage, d) != 0)
+        /* Add to the crate of all records */
+
+        x = listing_insert(&all_crate->listing, d);
+        if (x == NULL)
             return -1;
 
-        /* Add to crates */
+        /* If there is an existing entry, use it instead */
 
-        if (crate_add(all_crate, d) != 0)
-            return -1;
-        if (crate_add(crate, d) != 0)
-            return -1;
+        if (x != d) {
+            record_clear(d);
+            free(d);
+            d = x;
+        }
+
+        /* Insert into the user's crate */
+
+        if (sort) {
+            if (listing_insert(&crate->listing, d) == NULL)
+                return -1;
+        } else {
+            if (listing_add(&crate->listing, d) == -1)
+                return -1;
+        }
     }
 
     if (fclose(fp) == -1) {
@@ -324,10 +417,5 @@ int library_import(struct library_t *li, const char *scan, const char *path)
         return -1;
     }
 
-    /* sort the listings */
-
-    listing_sort(&all_crate->listing);
-    listing_sort(&crate->listing);
-
     return 0;
 }
