@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -148,7 +148,7 @@ static ssize_t pollfds(struct device_t *dv, struct pollfd *pe, size_t z)
         return -1;
     
     pe->fd = oss->fd;
-    pe->events = POLLIN | POLLOUT | POLLHUP;
+    pe->events = POLLIN | POLLOUT;
     oss->pe = pe;
 
     return 1;
