@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -23,17 +23,17 @@
 /* Values for the filter concluded experimentally */
 
 #define ALPHA (1.0/512)
-#define BETA (ALPHA/1024)
+#define BETA (ALPHA/256)
 
 /* State of the pitch calculation filter */
 
 struct pitch_t {
-    float dt, x, v;
+    double dt, x, v;
 };
 
 /* Prepare the filter for observations every dt seconds */
 
-static inline void pitch_init(struct pitch_t *p, float dt)
+static inline void pitch_init(struct pitch_t *p, double dt)
 {
     p->dt = dt;
     p->x = 0.0;
@@ -46,9 +46,9 @@ static inline void pitch_init(struct pitch_t *p, float dt)
  * Because the vinyl uses timestamps, the values for dx are discrete
  * rather than smooth. */
 
-static inline void pitch_dt_observation(struct pitch_t *p, float dx)
+static inline void pitch_dt_observation(struct pitch_t *p, double dx)
 {
-    float predicted_x, predicted_v, residual_x;
+    double predicted_x, predicted_v, residual_x;
 
     predicted_x = p->x + p->v * p->dt;
     predicted_v = p->v;
@@ -63,7 +63,8 @@ static inline void pitch_dt_observation(struct pitch_t *p, float dx)
 
 /* Get the pitch after filtering */
 
-static inline float pitch_current(struct pitch_t *p) {
+static inline double pitch_current(struct pitch_t *p)
+{
     return p->v;
 }
 
