@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,13 +20,14 @@
 #ifndef PLAYER_H
 #define PLAYER_H
 
+#include <stdbool.h>
+
 #include "track.h"
 
-#define PLAYER_RATE 44100
 #define PLAYER_CHANNELS 2
 
 struct player_t {
-    int reconnect; /* Re-sync the offset at next opportunity */
+    struct track_t *track;
 
     /* Current playback parameters */
 
@@ -38,25 +39,25 @@ struct player_t {
         sync_pitch, /* pitch required to sync to timecode signal */
         volume;
 
-    int target_valid;
+    bool target_valid;
+
+    /* Timecode control */
 
-    struct track_t *track;
     struct timecoder_t *timecoder;
+    bool timecode_control,
+        recalibrate; /* re-sync offset at next opportunity */
 };
 
-void player_init(struct player_t *pl);
+void player_init(struct player_t *pl, struct track_t *track);
 void player_clear(struct player_t *pl);
 
 void player_connect_timecoder(struct player_t *pl, struct timecoder_t *tc);
-void player_disconnect_timecoder(struct player_t *pl);
+void player_set_timecode_control(struct player_t *pl, bool on);
+bool player_toggle_timecode_control(struct player_t *pl);
 
-int player_control(struct player_t *pl, float pitch, float volume,
-                   int timecode_known, double target_position);
 int player_recue(struct player_t *pl);
 
 void player_collect(struct player_t *pl, signed short *pcm,
                     int samples, int rate);
 
-void player_connect_track(struct player_t *pl, struct track_t *tr);
-
 #endif
