@@ -1,15 +1,15 @@
 /*
- * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * version 2, as published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License version 2 for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * version 2 along with this program; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -31,6 +31,7 @@
 #include "library.h"
 #include "oss.h"
 #include "player.h"
+#include "realtime.h"
 #include "rig.h"
 #include "timecoder.h"
 #include "track.h"
@@ -49,60 +50,14 @@
 #define DEFAULT_SCANNER EXECDIR "/xwax-scan"
 #define DEFAULT_TIMECODE "serato_2a"
 
-
-/* We don't use the full flexibility of a rig, and just have a direct
- * correspondence between a device, track, player and timecoder */
-
-struct deck_t {
-    struct device_t device;
-    struct track_t track;
-    struct player_t player;
-    struct timecoder_t timecoder;
-};
-
-
-static int deck_init(struct deck_t *deck, const char *timecode, double speed,
-                     const char *importer, unsigned int sample_rate)
-{
-    if (timecoder_init(&deck->timecoder, timecode, speed, sample_rate) == -1)
-        return -1;
-    track_init(&deck->track, importer);
-    player_init(&deck->player);
-    player_connect_track(&deck->player, &deck->track);    
-    return 0;
-}
-
-
-static void deck_clear(struct deck_t *deck)
-{
-    track_clear(&deck->track);
-    timecoder_clear(&deck->timecoder);
-    player_clear(&deck->player);
-    device_clear(&deck->device);
-}
-
-
-static void connect_deck_to_interface(struct interface_t *iface, int n,
-                                      struct deck_t *deck)
-{
-    iface->timecoder[n] = &deck->timecoder;
-    iface->player[n] = &deck->player;
-}
-
-
-static void connect_deck_to_rig(struct rig_t *rig, int n, struct deck_t *deck)
-{
-    rig->device[n] = &deck->device;
-    rig->track[n] = &deck->track;
-    rig->player[n] = &deck->player;
-    rig->timecoder[n] = &deck->timecoder;
-}
-
+char *banner = "xwax " VERSION \
+    " (C) Copyright 2011 Mark Hills <mark@pogo.org.uk>";
 
 static void usage(FILE *fd)
 {
     fprintf(fd, "Usage: xwax [<options>]\n\n"
-      "  -l <directory> Directory to scan for audio tracks\n"
+      "  -l <path>      Location to scan for audio tracks\n"
+      "  -p <path>      Ordered playlist for audio tracks\n"
       "  -t <name>      Timecode name\n"
       "  -33            Use timecode at 33.3RPM (default)\n"
       "  -45            Use timecode at 45RPM\n"
@@ -139,28 +94,33 @@ static void usage(FILE *fd)
       "Decks and audio directories can be specified multiple times.\n\n"
       "Available timecodes (for use with -t):\n"
       "  serato_2a (default), serato_2b, serato_cd,\n"
-      "  traktor_a, traktor_b, mixvibes_v2, mixvibes_7inch\n\n");
+      "  traktor_a, traktor_b, mixvibes_v2, mixvibes_7inch\n\n"
+      "See the xwax(1) man page for full information and examples.\n");
 }
 
-
 int main(int argc, char *argv[])
 {
     int r, n, decks, oss_fragment, oss_buffers, rate, alsa_buffer;
     char *endptr, *timecode, *importer, *scanner;
     double speed;
 
-    struct deck_t deck[MAX_DECKS];
+    struct device_t device[MAX_DECKS];
+    struct track_t track[MAX_DECKS];
+    struct player_t player[MAX_DECKS];
+    struct timecoder_t timecoder[MAX_DECKS];
+
     struct rig_t rig;
+    struct rt_t rt;
     struct interface_t iface;
     struct library_t library;
-    struct device_t *device;
-    
-    fprintf(stderr, BANNER "\n\n" NOTICE "\n\n");
-    
-    interface_init(&iface);
-    rig_init(&rig);
+
+    fprintf(stderr, "%s\n\n" NOTICE "\n\n", banner);
+
+    if (rig_init(&rig) == -1)
+        return -1;
+    rt_init(&rt);
     library_init(&library);
-    
+
     decks = 0;
     oss_fragment = DEFAULT_OSS_FRAGMENT;
     oss_buffers = DEFAULT_OSS_BUFFERS;
@@ -172,10 +132,10 @@ int main(int argc, char *argv[])
     speed = 1.0;
 
     /* Skip over command name */
-    
+
     argv++;
     argc--;
-    
+
     while (argc > 0) {
 
         if (!strcmp(argv[0], "-h")) {
@@ -186,7 +146,7 @@ int main(int argc, char *argv[])
         } else if (!strcmp(argv[0], "-f")) {
 
             /* Set fragment size for subsequent devices */
-            
+
             if (argc < 2) {
                 fprintf(stderr, "-f requires an integer argument.\n");
                 return -1;
@@ -206,29 +166,29 @@ int main(int argc, char *argv[])
                         DEFAULT_OSS_FRAGMENT);
                 return -1;
             }
-            
+
             argv += 2;
             argc -= 2;
 
         } else if (!strcmp(argv[0], "-b")) {
-            
+
             /* Set number of buffers for subsequent devices */
-            
+
             if (argc < 2) {
                 fprintf(stderr, "-b requires an integer argument.\n");
                 return -1;
             }
-            
+
             oss_buffers = strtol(argv[1], &endptr, 10);
             if (*endptr != '\0') {
                 fprintf(stderr, "-b requires an integer argument.\n");
                 return -1;
             }
-            
+
             argv += 2;
             argc -= 2;
 #endif
-            
+
         } else if (!strcmp(argv[0], "-r")) {
 
             /* Set sample rate for subsequence devices */
@@ -245,32 +205,32 @@ int main(int argc, char *argv[])
             }
 
             argv += 2;
-            argc -= 2;  
+            argc -= 2;
 
 #ifdef WITH_ALSA
         } else if (!strcmp(argv[0], "-m")) {
-            
+
             /* Set size of ALSA buffer for subsequence devices */
-            
+
             if (argc < 2) {
                 fprintf(stderr, "-m requires an integer argument.\n");
                 return -1;
             }
-            
+
             alsa_buffer = strtol(argv[1], &endptr, 10);
             if (*endptr != '\0') {
                 fprintf(stderr, "-m requires an integer argument.\n");
                 return -1;
             }
-            
+
             argv += 2;
             argc -= 2;
 #endif
-            
+
         } else if (!strcmp(argv[0], "-d") || !strcmp(argv[0], "-a") ||
 		  !strcmp(argv[0], "-j"))
 	{
-	    unsigned int sample_rate;
+            unsigned int sample_rate;
 
             /* Create a deck */
 
@@ -285,29 +245,28 @@ int main(int argc, char *argv[])
                         MAX_DECKS);
                 return -1;
             }
-            
+
             fprintf(stderr, "Initialising deck %d (%s)...\n", decks, argv[1]);
 
             /* Work out which device type we are using, and initialise
              * an appropriate device. */
 
-            device = &deck[decks].device;
-
             switch(argv[0][1]) {
 
 #ifdef WITH_OSS
             case 'd':
-                r = oss_init(device, argv[1], rate, oss_buffers, oss_fragment);
+                r = oss_init(&device[decks], argv[1], rate,
+                             oss_buffers, oss_fragment);
                 break;
 #endif
 #ifdef WITH_ALSA
             case 'a':
-                r = alsa_init(device, argv[1], rate, alsa_buffer);
+                r = alsa_init(&device[decks], argv[1], rate, alsa_buffer);
                 break;
 #endif
 #ifdef WITH_JACK
             case 'j':
-                r = jack_init(device, argv[1]);
+                r = jack_init(&device[decks], argv[1]);
                 break;
 #endif
             default:
@@ -321,26 +280,29 @@ int main(int argc, char *argv[])
 
 	    sample_rate = device_sample_rate(device);
 
-            if (deck_init(&deck[decks], timecode, speed,
-                          importer, sample_rate) == -1)
+            if (timecoder_init(&timecoder[decks], timecode,
+                               speed, sample_rate) == -1)
             {
                 return -1;
             }
 
+            if (rt_add_device(&rt, &device[decks]) == -1)
+                return -1;
+
+            track_init(&track[decks], importer);
+            rig_add_track(&rig, &track[decks]);
+
+            player_init(&player[decks], &track[decks]);
+            player_connect_timecoder(&player[decks], &timecoder[decks]);
+
             /* The timecoder and player are driven by requests from
              * the audio device */
-            
-            device_connect_timecoder(device, &deck[decks].timecoder);
-            device_connect_player(device, &deck[decks].player);
 
-            /* The rig and interface keep track of everything whilst
-             * the program is running */
-
-            connect_deck_to_interface(&iface, decks, &deck[decks]);
-            connect_deck_to_rig(&rig, decks, &deck[decks]);
+            device_connect_timecoder(&device[decks], &timecoder[decks]);
+            device_connect_player(&device[decks], &player[decks]);
 
             decks++;
-            
+
             argv += 2;
             argc -= 2;
 
@@ -354,7 +316,7 @@ int main(int argc, char *argv[])
             }
 
             timecode = argv[1];
-            
+
             argv += 2;
             argc -= 2;
 
@@ -401,17 +363,26 @@ int main(int argc, char *argv[])
 
             argv += 2;
             argc -= 2;
-                        
-        } else if (!strcmp(argv[0], "-l")) {
+
+        } else if (!strcmp(argv[0], "-l") || !strcmp(argv[0], "-p")) {
+
+            bool sort;
 
             /* Load in a music library */
 
             if (argc < 2) {
-                fprintf(stderr, "-l requires a pathname as an argument.\n");
+                fprintf(stderr, "-%c requires a pathname as an argument.\n",
+                        argv[0][1]);
                 return -1;
             }
 
-            if (library_import(&library, scanner, argv[1]) == -1)
+            if (argv[0][1] == 'p') {
+                sort = false;
+            } else {
+                sort = true;
+            }
+
+            if (library_import(&library, sort, scanner, argv[1]) == -1)
                 return -1;
 
             argv += 2;
@@ -429,35 +400,32 @@ int main(int argc, char *argv[])
         return -1;
     }
 
-    iface.players = decks;
-    iface.timecoders = decks;
+    if (rt_start(&rt) == -1)
+        return -1;
+    if (interface_start(&iface, decks, player, timecoder, &library, &rig) == -1)
+        return -1;
 
-    /* Connect everything up. Do this after selecting a timecode and
-     * built the lookup tables. */
-
-    for (n = 0; n < decks; n++)
-        player_connect_timecoder(&deck[n].player, &deck[n].timecoder);
-
-    fprintf(stderr, "Starting threads...\n");
-    if (rig_start(&rig) == -1)
+    if (rig_main(&rig) == -1)
         return -1;
 
-    fprintf(stderr, "Entering interface...\n");
-    iface.library = &library;
-    interface_run(&iface);
-    
     fprintf(stderr, "Exiting cleanly...\n");
 
-    if (rig_stop(&rig) == -1)
-        return -1;
-    
-    for (n = 0; n < decks; n++)
-        deck_clear(&deck[n]);
-    
+    interface_stop(&iface);
+    rt_stop(&rt);
+
+    for (n = 0; n < decks; n++) {
+        track_clear(&track[n]);
+        timecoder_clear(&timecoder[n]);
+        player_clear(&player[n]);
+        device_clear(&device[n]);
+    }
+
     timecoder_free_lookup();
     library_clear(&library);
-    
+    rt_clear(&rt);
+    rig_clear(&rig);
+
     fprintf(stderr, "Done.\n");
-    
+
     return 0;
 }
