@@ -1,15 +1,15 @@
 /*
- * Copyright (C) 2010 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2011 Mark Hills <mark@pogo.org.uk>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * version 2, as published by the Free Software Foundation.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  * General Public License version 2 for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * version 2 along with this program; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
@@ -20,7 +20,7 @@
 #ifndef XWAX_H
 #define XWAX_H
 
-#define BANNER "xwax 0.8 (C) Copyright 2010 Mark Hills <mark@pogo.org.uk>"
+extern char *banner;
 
 #define NOTICE \
   "This software is supplied WITHOUT ANY WARRANTY; without even the implied\n"\
